"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1MessagePickupProtocol = void 0;
const models_1 = require("../../../../agent/models");
const constants_1 = require("../../../../constants");
const error_1 = require("../../../../error");
const plugins_1 = require("../../../../plugins");
const MessagePickupModuleConfig_1 = require("../../MessagePickupModuleConfig");
const BaseMessagePickupProtocol_1 = require("../BaseMessagePickupProtocol");
const handlers_1 = require("./handlers");
const messages_1 = require("./messages");
let V1MessagePickupProtocol = class V1MessagePickupProtocol extends BaseMessagePickupProtocol_1.BaseMessagePickupProtocol {
    constructor() {
        super(...arguments);
        /**
         * The version of the message pickup protocol this class supports
         */
        this.version = 'v1';
    }
    /**
     * Registers the protocol implementation (handlers, feature registry) on the agent.
     */
    register(dependencyManager, featureRegistry) {
        dependencyManager.registerMessageHandlers([new handlers_1.V1BatchPickupHandler(this), new handlers_1.V1BatchHandler()]);
        featureRegistry.register(new models_1.Protocol({
            id: 'https://didcomm.org/messagepickup/1.0',
            roles: ['message_holder', 'recipient', 'batch_sender', 'batch_recipient'],
        }));
    }
    async createPickupMessage(agentContext, options) {
        const { connectionRecord, batchSize } = options;
        connectionRecord.assertReady();
        const config = agentContext.dependencyManager.resolve(MessagePickupModuleConfig_1.MessagePickupModuleConfig);
        const message = new messages_1.V1BatchPickupMessage({
            batchSize: batchSize !== null && batchSize !== void 0 ? batchSize : config.maximumBatchSize,
        });
        return { message };
    }
    async createDeliveryMessage(agentContext, options) {
        const { connectionRecord, batchSize, messages } = options;
        connectionRecord.assertReady();
        const pickupMessageQueue = agentContext.dependencyManager.resolve(constants_1.InjectionSymbols.MessagePickupRepository);
        const messagesToDeliver = messages !== null && messages !== void 0 ? messages : (await pickupMessageQueue.takeFromQueue({
            connectionId: connectionRecord.id,
            limit: batchSize, // TODO: Define as config parameter for message holder side
        }));
        const batchMessages = messagesToDeliver.map((msg) => new messages_1.BatchMessageMessage({
            id: msg.id,
            message: msg.encryptedMessage,
        }));
        if (messagesToDeliver.length > 0) {
            const message = new messages_1.V1BatchMessage({
                messages: batchMessages,
            });
            return { message };
        }
    }
    async setLiveDeliveryMode() {
        throw new error_1.CredoError('Live Delivery mode not supported in Message Pickup V1 protocol');
    }
    async processBatchPickup(messageContext) {
        // Assert ready connection
        const connection = messageContext.assertReadyConnection();
        const { message } = messageContext;
        const pickupMessageQueue = messageContext.agentContext.dependencyManager.resolve(constants_1.InjectionSymbols.MessagePickupRepository);
        const messages = await pickupMessageQueue.takeFromQueue({
            connectionId: connection.id,
            limit: message.batchSize,
            deleteMessages: true,
        });
        const batchMessages = messages.map((msg) => new messages_1.BatchMessageMessage({
            id: msg.id,
            message: msg.encryptedMessage,
        }));
        const batchMessage = new messages_1.V1BatchMessage({
            messages: batchMessages,
        });
        return new models_1.OutboundMessageContext(batchMessage, { agentContext: messageContext.agentContext, connection });
    }
};
V1MessagePickupProtocol = __decorate([
    (0, plugins_1.injectable)()
], V1MessagePickupProtocol);
exports.V1MessagePickupProtocol = V1MessagePickupProtocol;
//# sourceMappingURL=V1MessagePickupProtocol.js.map